var AutomationModCollection = {
	name: 'AutomationModCollection',
	version: '1.5',
	gameVersion: '2.048',
	isLoaded: false,
	config: {
		autoClicker: 1,
		autoClickerInterval: 10,
		showCycles: 0,
		efficiency: 1,
		clickGolden: 1,
		clickReindeer: 1,
		clickWrath: 0,
		clickFortune: 1,
		useChocolateEgg: 0,
	},
	init: function() {
		if(CCSE.isLoaded) {
			Game.customOptionsMenu.push(this.addOptionsMenu);
			this.insertText();

			/*
				Register Efficiency Hook from Daniels Efficiency Mod
				- Check and Update Efficiency
				- Print Efficiency
			*/
			Game.registerHook('logic', () => {
				this.updateEfficiency();
			});

			/*
				Register CookieCycler
				- Auto clicking
				- Clickcalls per second printing
			*/
			if(this.config.autoClicker === 1) {
				Game.Notify(`Auto Clicker is Loaded!`,'',[16,5]);
				this.cookieCycler.init(this);
			} else {
				Game.Notify(`Auto Clicker is Disabled!`,'',[16,5]);
			}

			/*
				Setup the Logic Loop
			*/
			setInterval(() => {
				this.logicCycle();
			}, 150);
			this.isLoaded = true;
		}
	},
	register: function() {
		if(CCSE.ConfirmGameVersion(this.name, this.version, this.gameVersion)) {
			Game.registerMod(this.name, this);
		}
	},
	logicCycle: function() {
		/*
			Register Cycler for Clicking Golden Cookies and Reindeers
			- Clicks Golden Cookies if set to 1 in config
			- Clicks Reindeers if set to 1 in config
		*/
		if(this.config.clickGolden == 1 || this.config.clickReindeer == 1) {
			this.clickGoldenCookie();
		}

		/*
			Register Cycler for Clicking Fortune News
			- Clicks Fortune news -> Untested
		*/
		if(this.config.clickFortune == 1) {
			this.clickFortuneCookie();
		}

		/*
			Placeholder functionality untill later
		*/
		if(this.config.useChocolateEgg == 1) {
			this.useChocolateEgg();
		}
	},
	insertText: function() {
		Game.ObjectsById.forEach(obj => {
			obj
				.l.getElementsByClassName("content")[0]
				.insertAdjacentHTML('beforeend', '<div class="efficiencyMod" style="position: absolute;top: 0px;right: 0px;"><span class="efficiency" style="vertical-align: top;">0%</span></div>');
		});
		// if(this.config.showCycles == 1) {
		// 	l("sectionLeft").insertAdjacentHTML('beforeend', `<div id="cpsCounter" style="position: absolute; bottom: 10vh; left: 0; transfrom: translateX(-50%); font-size: 1em; font-weight: bold; text-shadow: 0 0 0.2vh #fff; box-shadow: 0 0 3vh #111a, 0 0 3vh #111a, 0 0 3vh #111a; width: 30vw; text-align: center; padding: 1vh 0; background-color: #111a; cursor-event: none; z-index: 99;">Clicks p/s last Cycle: 0 - Clicks p/s new Cycle: 0</div>`);
		// }
	},
	cookieCycler: {
		currentCallCount: 0, // The amount of Function calls in the current Cycle
		lastCycle: 0, // The amount of Function calls in the 2nd last Cycle
		currentCycle: 0, // the amount of Function calls in the last Cycle
		init(gameInstance) {
			/*
				Main Interval calling the ClickCookie().
				Interval timeout - 1ms
			*/
			setInterval(() => {
				Game.ClickCookie();
				this.currentCallCount++;
			}, gameInstance.config.autoClickerInterval);
			/*
				Secondary Interval printing the calls in the past 1000ms.
				Interval timeout - 1000ms | 1s
			*/
			setInterval(() => {
				this.setCurrentCycle(this.currentCallCount);
				this.currentCallCount = 0;
			}, 1000);
		},
		setCurrentCycle(value) {
			this.lastCycle = this.currentCycle;
			this.currentCycle = value;
			/*
				Update Counter for User
			*/
			l(`cpsCounter`).innerText = `Clicks p/s last Cycle: ${this.lastCycle} - Clicks p/s new Cycle: ${this.currentCycle}`;
		},
	},
	clickGoldenCookie: function() {
		for(const shimmer of Game.shimmers) {
			if (shimmer.type === 'golden') {
				if(this.config.clickGolden && (!shimmer.wrath || shimmer.force === 'cookie storm drop' || this.config.clickWrath)) {
					shimmer.pop();
				}
			} else if (this.config.clickReindeer && shimmer.type === 'reindeer') {
				shimmer.pop();
			}
		}
	},
	clickFortuneCookie: function() {
		if(Game.TickerEffect && Game.TickerEffect.type === 'fortune') {
			Game.tickerL.click();
		}
	},
	useChocolateEgg: function() {
		if (this.config.useChocolateEgg && Game.Has('Inspired checklist') && !Game.Upgrades['Chocolate egg'].isVaulted()){
			Game.Upgrades['Chocolate egg'].vault();
			Game.upgradesToRebuild=1;
		}
	},
	updateEfficiency: function () {
		Game.ObjectsById.forEach(obj => {
			var efficiency = this.getEfficiency(obj);
			var efficiencyDiv = obj.l.getElementsByClassName("content")[0].getElementsByClassName("efficiency")[0];
			efficiencyDiv.innerHTML = '<b>' + Beautify(efficiency, 1) + '</b>';
		});
	},
	getEfficiency: function(me) {
		//Synergy stuff
		//Copied and modified from game main.js line 7845
		var synergyBoost = 0;
		if(me.amount > 0) {
			if(me.name == 'Grandma') {
				for(var i in Game.GrandmaSynergies) {
					if(Game.Has(Game.GrandmaSynergies[i]))	{
						var other = Game.Upgrades[Game.GrandmaSynergies[i]].buildingTie;
						var multOriginal = me.amount * 0.01 * (1 / (other.id - 1));
						var boostOriginal = (other.storedTotalCps * Game.globalCpsMult) - (other.storedTotalCps * Game.globalCpsMult) / (1 + multOriginal);
						// + 1 as we need to work out what it is if we add another of this building
						var multNew = (me.amount + 1) * 0.01 * (1 / (other.id - 1));
						var boostNew = (other.storedTotalCps * Game.globalCpsMult) - (other.storedTotalCps * Game.globalCpsMult) / (1 + multNew);

						//Boost is the difference between them
						synergyBoost+=boostNew - boostOriginal;
					}
				}
			} else if(me.name == 'Portal' && Game.Has('Elder Pact')) {
				var other=Game.Objects['Grandma'];
				var boostOriginal=(me.amount*0.05*other.amount)*Game.globalCpsMult;
				// + 1 as we need to work out what it is if we add another of this building
				var boostNew=((me.amount+1)*0.05*other.amount)*Game.globalCpsMult;

				//Boost is the difference between them
				synergyBoost+=boostNew - boostOriginal;
			}

			for(var i in me.synergies)	{
				var it = me.synergies[i];
				if(Game.Has(it.name)) {
					var weight = 0.05;
					var other = it.buildingTie1;
					if (me == it.buildingTie1) {
						weight = 0.001;
						other = it.buildingTie2;
					}
					var boostOriginal = (other.storedTotalCps * Game.globalCpsMult) - (other.storedTotalCps * Game.globalCpsMult) / (1 + me.amount * weight);
					// + 1 as we need to work out what it is if we add another of this building
					var boostNew = (other.storedTotalCps * Game.globalCpsMult) - (other.storedTotalCps * Game.globalCpsMult) / (1 + (me.amount + 1) * weight);

					//Boost is the difference between them
					synergyBoost += boostNew - boostOriginal;
				}
			}
		}

		//Main efficiency calculation + synergy
		var efficiency = me.amount > 0 ? me.bulkPrice / (((me.storedTotalCps / me.amount) * Game.globalCpsMult) + synergyBoost) : me.bulkPrice / ((me.baseCps * Game.globalCpsMult) + synergyBoost);
		return efficiency;
	},
	addOptionsMenu: function() {
        const body = `
        <div class="listing" style="padding: 5px 16px; opacity: 0.7; font-size: 17px; font-family: Kavoon, Georgia, serif;">Automation Setup</div>
        <div class="listing">
            ${this.button('clickGolden', 'Click Golden Cookies ON', 'Click Golden Cookies OFF')}
		</div>
		<div class="listing">
            ${this.button('clickWrath', 'Mode: Include Wrath Cookies', 'Mode: Exclude Wrath Cookies')}
		</div>
		<div class="listing">
            ${this.button('clickReindeer', 'Click Reindeers ON', 'Click Reindeer OFF')}
        </div>
		<div class="listing">
            ${this.button('clickFortune', 'Click Fortune Tickers ON', 'Click Fortune Tickers OFF')}
        </div>
		<div class="listing">
			${this.button('showCycles (Works after restart - Save before restarting)', 'ShowCycles ON', 'ShowCycles OFF')}
		</div>`;

        CCSE.AppendCollapsibleOptionsMenu('AutomationCollection Config', body)
    },
	button: function(config, textOn, textOff) {
        const name = `AutomationModCollection-${config}`;
        const callback = `AutomationModCollection.configCallback('${config}', '${name}', '${textOn}', '${textOff}');`
        const value = this.config[config];
        return `<a class="${value ? 'option' : 'option off'}" id="${name}" style="font-size: 12px;" ${Game.clickStr}="${callback}">${value ? textOn : textOff}</a>`
    },
    configCallback: function(config, button, textOn, textOff) {
        const value = !this.config[config];
        this.config[config] = value;
        l(button).innerHTML = value ? textOn : textOff
        l(button).className = value ? 'option' : 'option off'
    },
	load: function(save) {
		try {
			const config = JSON.parse(save);
			Object.assign(this.config, config);
		} catch(error) {
			Game.Notify(`Error occured loading config!`,'',[16,5]);
		}
	},
	save: function() {
		return JSON.stringify(this.config);
	},
};

for(func of Object.getOwnPropertyNames(AutomationModCollection).filter(m => typeof AutomationModCollection[m] === 'function')) {
    AutomationModCollection[func] = AutomationModCollection[func].bind(AutomationModCollection);
}

// Load mod
if(!AutomationModCollection.isLoaded) {
	if(CCSE && CCSE.isLoaded) {
		AutomationModCollection.register();
	} else {
		if(!CCSE) {
			var CCSE = {};
		}
		if(!CCSE.postLoadHooks) {
			CCSE.postLoadHooks = [];
		}
		CCSE.postLoadHooks.push(AutomationModCollection.register);
	}
}
